
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.util.List;

/**
 * web temperature setter
 *
 * @author matecsaba
 */
public class temper implements Runnable {

    /**
     * this is needed for cli startup
     *
     * @param args command line parameters
     */
    public static void main(String[] args) {
        temper app = new temper();
        String a;
        try {
            ByteArrayOutputStream buf = new ByteArrayOutputStream();
            a = "" + app.getClass().getName();
            a = app.httpRequest("http://localhost/" + a, "./" + a, "cli", "clibrowser", args, buf);
            a = "type=" + a + "\r\ndata:\r\n" + buf.toString();
        } catch (Exception e) {
            a = "exception " + e.getMessage();
        }
        System.out.println(a);
    }

    /**
     * where i'm located in ram
     */
    protected static temper staticTemper = null;

    /**
     * do one request
     *
     * @param url url of app
     * @param path path of app
     * @param peer client address
     * @param agent user agent
     * @param par parameters
     * @param buf result buffer, if empty, pathname must present
     * @return [pathname"][file name.]extension
     * @throws Exception if something went wrong
     */
    public static String httpRequest(String url, String path, String peer,
            String agent, String[] par, ByteArrayOutputStream buf)
            throws Exception {
        url = new URL(url).getPath();
        if (staticTemper == null) {
            staticTemper = new temper();
            staticTemper.path = path.substring(0, path.lastIndexOf("."));
            staticTemper.url = url;
            new Thread(staticTemper).start();
        }
        staticTemper.doRequest(par, buf, peer);
        return "html";
    }

    /**
     * where i'm located on host
     */
    protected String path;

    /**
     * where i'm located on net
     */
    protected String url;

    /**
     * current value: 0=standby, 1=heating, 2=door
     */
    protected int currValue = 0x10000;

    /**
     * last measured temperature
     */
    protected int lastMeasure;

    /**
     * last needed temperature
     */
    protected int lastNeeded;

    /**
     * last window temperature
     */
    protected int lastWindow;

    /**
     * time measured temperature
     */
    protected long timeMeasure;

    /**
     * time needed temperature
     */
    protected long timeNeeded;

    /**
     * time heating temperature
     */
    protected long timeHeating;

    /**
     * time window temperature
     */
    protected long timeWindow;

    /**
     * log file
     */
    protected String logFile;

    /**
     * door code
     */
    protected String doorCode = "";

    /**
     * door time
     */
    protected int doorTime = 100;

    /**
     * minimum value
     */
    protected int minValue = 10;

    /**
     * maximum value
     */
    protected int maxValue = 30;

    /**
     * last setter peer
     */
    protected String lastSetter = "<nobody>";

    private synchronized void setValue(int val) {
        if (currValue == val) {
            return;
        }
        currValue = val;
        timeHeating = temperUtil.getTime();
        try {
            Runtime rtm = Runtime.getRuntime();
            String[] cmd = new String[2];
            cmd[0] = "./tempset.sh";
            cmd[1] = val + "";
            Process prc = rtm.exec(cmd);
            prc.waitFor();
        } catch (Exception e) {
        }
    }

    private synchronized void getValue() {
        try {
            Runtime rtm = Runtime.getRuntime();
            String[] cmd = new String[1];
            cmd[0] = "./tempget.sh";
            Process prc = rtm.exec(cmd);
            prc.waitFor();
            int i = prc.exitValue();
            if (i < 1) {
                return;
            }
            lastMeasure = i;
            timeMeasure = temperUtil.getTime();
        } catch (Exception e) {
        }
    }

    private synchronized void writeLog(String who) {
        if (who == null) {
            who = lastSetter;
        }
        temperUtil.append(logFile, temperUtil.getTime() + ";" + timeMeasure + ";" + timeWindow + ";" + who + ";" + currValue + ";" + lastNeeded + ";" + lastWindow + ";" + lastMeasure);
    }

    private int doCalc() {
        long tim = temperUtil.getTime();
        int val = currValue & 1;
        int old = val;
        if (lastNeeded > lastMeasure) {
            val = 1;
        }
        if (lastNeeded < lastMeasure) {
            val = 0;
        }
        if ((tim - timeMeasure) > (5 * 60 * 1000)) {
            val = 0;
        }
        if (timeWindow > 0) {
            if (lastMeasure < lastWindow) {
                lastWindow = lastMeasure;
                timeWindow = tim;
            }
            if ((tim - timeWindow) > (30 * 60 * 1000)) {
                timeWindow = 0;
            }
            val = 0;
        }
        if ((old == 1) && (lastMeasure < lastWindow)) {
            timeWindow = tim;
            val = 0;
        }
        if ((old == 0) && (val == 1)) {
            lastWindow = lastMeasure;
        }
        return val | (currValue & 2);
    }

    public void run() {
        logFile = path + ".log";
        lastMeasure = 20;
        readConfig();
        getValue();
        lastNeeded = lastMeasure;
        lastWindow = lastMeasure;
        rangeCheck();
        setValue(doCalc());
        writeLog("<boot>");
        for (;;) {
            temperUtil.sleep(60 * 1000);
            getValue();
            rangeCheck();
            setValue(doCalc());
            writeLog(null);
        }
    }

    private void readConfig() {
        List<String> c = temperUtil.readup(path + ".cfg");
        if (c == null) {
            return;
        }
        if (c.size() < 1) {
            return;
        }
        doorCode = c.get(0);
        doorTime = temperUtil.str2num(c.get(1));
        logFile = c.get(2);
        minValue = temperUtil.str2num(c.get(3));
        maxValue = temperUtil.str2num(c.get(4));
    }

    private void rangeCheck() {
        if (lastNeeded > maxValue) {
            lastNeeded = maxValue;
        }
        if (lastNeeded < minValue) {
            lastNeeded = minValue;
        }
    }

    /**
     * do one request
     *
     * @param par parameters
     * @param buf buffer to use
     * @param peer address
     * @return 1 on html result
     * @throws Exception on error
     */
    public synchronized int doRequest(String[] par, ByteArrayOutputStream buf, String peer) throws Exception {
        String tmp = "";
        String cmd = "";
        for (int i = 0; i < par.length; i++) {
            String a = par[i];
            int o = a.indexOf("=");
            if (o < 1) {
                continue;
            }
            String b = a.substring(0, o);
            a = a.substring(o + 1, a.length());
            if (b.equals("temp")) {
                tmp = a;
            }
            if (b.equals("cmd")) {
                cmd = a;
            }
        }
        if (cmd.equals("heat")) {
            lastNeeded = temperUtil.str2num(tmp);
            timeNeeded = temperUtil.getTime();
            lastSetter = peer;
        }
        if (cmd.equals("door")) {
            String a = "bad code";
            if (tmp.equals(doorCode)) {
                setValue((currValue & 1) | 2);
                writeLog(peer);
                temperUtil.sleep(doorTime);
                setValue(currValue & 1);
                writeLog(peer);
                a = "door opened";
            }
            buf.write("<html><head><title>door</title><body>".getBytes());
            buf.write(a.getBytes());
            buf.write("</body></html>".getBytes());
            return 1;
        }
        rangeCheck();
        String a = "<html><head><title>temper</title>";
        buf.write(a.getBytes());
        a = "<meta http-equiv=refresh content=\"30;url=" + url + "\"></head>";
        buf.write(a.getBytes());
        a = "<body bgcolor=\"#000000\" text=\"#00FF00\" link=\"#00FFFF\" vlink=\"#00FFFF\" alink=\"#00FFFF\">";
        buf.write(a.getBytes());
        long tim = temperUtil.getTime();
        a = "measured: " + lastMeasure + " celsius, " + temperUtil.timePast(tim, timeMeasure) + " ago<br/>";
        buf.write(a.getBytes());
        a = "needed: " + lastNeeded + " celsius, " + temperUtil.timePast(tim, timeNeeded) + " ago by " + lastSetter + "<br/>";
        buf.write(a.getBytes());
        if (timeWindow > 0) {
            a = "window: " + lastWindow + " celsius, " + temperUtil.timePast(tim, timeWindow) + " ago<br/>";
            buf.write(a.getBytes());
        }
        a = "heating: " + currValue + ", " + temperUtil.timePast(tim, timeHeating) + " ago<br/>";
        buf.write(a.getBytes());
        a = "<form action=\"" + url + "\" method=get>wish: <input type=text name=temp value=\"" + lastNeeded + "\"> celsius";
        buf.write(a.getBytes());
        buf.write("<input type=submit name=cmd value=\"heat\">".getBytes());
        buf.write("</form><br/>".getBytes());
        for (int i = -5; i <= 5; i++) {
            int o = i + lastNeeded;
            a = "((<a href=\"" + url + "?temp=" + o + "&cmd=heat\">" + o + "</a>))";
            buf.write(a.getBytes());
        }
        buf.write("</body></html>".getBytes());
        return 1;
    }

}
